# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: models.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='models.proto',
  package='models',
  syntax='proto3',
  serialized_pb=_b('\n\x0cmodels.proto\x12\x06models\"V\n\x0eSessionContext\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07user_pk\x18\x02 \x01(\x04\x12\x13\n\x0bservice_due\x18\x03 \x01(\x04\x12\x12\n\ncreated_at\x18\x04 \x01(\x04\"\xd4\x01\n\x0b\x41uthContext\x12\x0b\n\x03uid\x18\x01 \x01(\t\x12\x0f\n\x07user_pk\x18\x02 \x01(\x04\x12\x11\n\tdevice_id\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65vice_type\x18\x04 \x01(\t\x12\x12\n\ndevice_mac\x18\x05 \x01(\t\x12\x13\n\x0b\x64\x65vice_imei\x18\x06 \x01(\t\x12\x10\n\x08platform\x18\x07 \x01(\t\x12\x18\n\x10platform_version\x18\x08 \x01(\t\x12\x16\n\x0e\x63lient_version\x18\t \x01(\t\x12\x12\n\nuser_agent\x18\n \x01(\tb\x06proto3')
)




_SESSIONCONTEXT = _descriptor.Descriptor(
  name='SessionContext',
  full_name='models.SessionContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='models.SessionContext.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_pk', full_name='models.SessionContext.user_pk', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='service_due', full_name='models.SessionContext.service_due', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='models.SessionContext.created_at', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=110,
)


_AUTHCONTEXT = _descriptor.Descriptor(
  name='AuthContext',
  full_name='models.AuthContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uid', full_name='models.AuthContext.uid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_pk', full_name='models.AuthContext.user_pk', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_id', full_name='models.AuthContext.device_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_type', full_name='models.AuthContext.device_type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_mac', full_name='models.AuthContext.device_mac', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_imei', full_name='models.AuthContext.device_imei', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='platform', full_name='models.AuthContext.platform', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='platform_version', full_name='models.AuthContext.platform_version', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='client_version', full_name='models.AuthContext.client_version', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_agent', full_name='models.AuthContext.user_agent', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=113,
  serialized_end=325,
)

DESCRIPTOR.message_types_by_name['SessionContext'] = _SESSIONCONTEXT
DESCRIPTOR.message_types_by_name['AuthContext'] = _AUTHCONTEXT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SessionContext = _reflection.GeneratedProtocolMessageType('SessionContext', (_message.Message,), dict(
  DESCRIPTOR = _SESSIONCONTEXT,
  __module__ = 'models_pb2'
  # @@protoc_insertion_point(class_scope:models.SessionContext)
  ))
_sym_db.RegisterMessage(SessionContext)

AuthContext = _reflection.GeneratedProtocolMessageType('AuthContext', (_message.Message,), dict(
  DESCRIPTOR = _AUTHCONTEXT,
  __module__ = 'models_pb2'
  # @@protoc_insertion_point(class_scope:models.AuthContext)
  ))
_sym_db.RegisterMessage(AuthContext)


# @@protoc_insertion_point(module_scope)
